#include	<windows.h>
#include	<malloc.h>
#include	<assert.h>
#include	<stdlib.h>
#include	<stdio.h>

#include	<winioctl.h>	//not devioctl.h as written in ezusbsys.h
#include	"ezusbsys.h"

#include	"main.h"

#include <winspool.h>
#include "ntddpar.h"

#define STATUS_INVALID_PARAMETER         (0xC000000DL)

#define		SUCCESS		TRUE
#define		FAILURE		FALSE

//#define	MAX_DRIVER_NAME	10	//defined in main.h

HANDLE  hDevice = NULL;
char    pcDriverName[MAX_DRIVER_NAME]; // = "Ezusb-0";
BOOLEAN	bResult;

int	nBytes = 0;
UCHAR	InterfaceInfo[1024];

char PIPE_TYPE_STRINGS[4][4] = 
{
"CTL",
"ISO",
"BLK",
"INT"
};

char PIPE_DIRECTION[2][4] = 
{
"OUT",
"IN "
};

//
// cut and pasted from USB100.h so this sample would
// not require the WDM DDK to build
//
typedef struct _USB_DEVICE_DESCRIPTOR {
    UCHAR bLength;
    UCHAR bDescriptorType;
    USHORT bcdUSB;
    UCHAR bDeviceClass;
    UCHAR bDeviceSubClass;
    UCHAR bDeviceProtocol;
    UCHAR bMaxPacketSize0;
    USHORT idVendor;
    USHORT idProduct;
    USHORT bcdDevice;
    UCHAR iManufacturer;
    UCHAR iProduct;
    UCHAR iSerialNumber;
    UCHAR bNumConfigurations;
} USB_DEVICE_DESCRIPTOR, *PUSB_DEVICE_DESCRIPTOR;

void
DumpDeviceDescriptor(PUSB_DEVICE_DESCRIPTOR deviceDescriptor);


/**************************************************
 * bOpenDriver proc                               *
 *                                                *
 * Purpose:                                       *  
 *      Opens the device driver using symbolic    *
 *      name provided                             *
 *                                                *
 * Input:                                         *
 *      phDeviceHandle:                           *
 *          Pointer to Device Driver handle where *
 *          the file handle is placed.            *
 *      devname:                                  *
 *          Null terminated string containing the *
 *          device name                           *
 *                                                *
 * Return Value:                                  *
 *      Boolean that indicates if the driver was  *
 *      successfully opened or not.               *
 *                                                *
 **************************************************/

BOOLEAN
bOpenDriver (HANDLE * phDeviceHandle, PCHAR devname)
{
    char completeDeviceName[64] = "";
    char pcMsg[64] = "";

    strcat (completeDeviceName,
            "\\\\.\\"
            );

    strcat (completeDeviceName,
		    devname
		    );

    *phDeviceHandle = CreateFile(   completeDeviceName,
		                            GENERIC_WRITE,
		                            FILE_SHARE_WRITE,
		                            NULL,
		                            OPEN_EXISTING,
		                            0,
		                            NULL);

    if (*phDeviceHandle == INVALID_HANDLE_VALUE) {
        return (FALSE);
    } else {
        return (TRUE);
    } /*else*/


}//OpenDevice

/*----------------------------------------------------------------------------+
|                                                                             |
|                         T R A N _ B U L K _ D A T A                         |
|                                                                             |
+-----------------------------------------------------------------------------*/

BOOLEAN Tran_Bulk_Data(char *buffer, ULONG buflen)

{

BULK_TRANSFER_CONTROL		bulkControl;		//ULONG pipeNum

DWORD	ioctl_val;
UINT	uiData = 0;
static	int		times_called = 0;

	times_called++;
                
    bulkControl.pipeNum = 0;   //want endpoint1 out
	if (hDevice == NULL )
		printf( "Tran_Bulk_Data failure, device not open\n" );

	if (hDevice != NULL)
    {

		ioctl_val = IOCTL_EZUSB_BULK_WRITE;
		bResult = DeviceIoControl (hDevice,
						ioctl_val, 
						&bulkControl,
						sizeof (BULK_TRANSFER_CONTROL),
						buffer,
						buflen,
						(unsigned long *)&nBytes,
						NULL);
					
		if( (bResult == TRUE) ) 
		{
			printf( "\rCompleted buffer #:%d", times_called );
		} 
		else 
		{
				printf( "Write IOCTL failed\n" );
		}/* else ioctl failed */
	}// if valid hDevice

	return( bResult );
}

/*----------------------------------------------------------------------------+
|                                                                             |
|                         R E C V _ B U L K _ D A T A                         |
|                                                                             |
+-----------------------------------------------------------------------------*/

BOOLEAN Recv_Bulk_Data(char *buffer, ULONG buflen)

{

BULK_TRANSFER_CONTROL		bulkControl;		//ULONG pipeNum

DWORD	ioctl_val;
UINT	uiData = 0;
static	int		times_called = 0;

	times_called++;
                
    bulkControl.pipeNum = 1;   //want endpoint1 in
	if (hDevice == NULL )
		printf( "Recv_Bulk_Data in failure, device not open\n" );

	if (hDevice != NULL)
    {

		ioctl_val = IOCTL_EZUSB_BULK_READ;
		bResult = DeviceIoControl (hDevice,
						ioctl_val, 
						&bulkControl,
						sizeof (BULK_TRANSFER_CONTROL),
						buffer,
						1, //buflen,
						(unsigned long *)&nBytes,
						NULL);
					
		if( (bResult == TRUE) ) 
		{
			printf( "Read IOCTL passed\n" );
			printf( "%d bytes read : 0x%02x\n", nBytes, buffer[0] );
		} 
		else 
		{
			printf( "Read IOCTL failed\n" );
		}/* else ioctl failed */
	}// if valid hDevice

	return( bResult );
}

/*----------------------------------------------------------------------------+
|                                                                             |
|                         S E N D _ F I L E _ D A T A                         |
|                                                                             |
+-----------------------------------------------------------------------------*/

Send_File_Data( char *filename ) 
{
FILE	*fp;
ULONG	buffer_char_count;		//usb needs buffer length to be ULONG
ULONG	total_char_count;
int		buffers_sent_count;
BOOLEAN	bulk_transfer_status;

int	c;

unsigned	char	buffer[64];

   if (! (fp = fopen( filename, "rb" )) )
   {
      printf( "Error opening data file %s\n", filename );
	  fclose( fp );
	  return -1;
   }
   else
   {
///	   printf( "file %s successfully opened\n", filename );
   }
   total_char_count = 0;
   buffer_char_count = 0;
   buffers_sent_count = 0;
   while ( (c = fgetc( fp )) != EOF ) 
   {
	   buffer[ buffer_char_count ] = (unsigned char) c;
	   buffer_char_count++;
	   if ( buffer_char_count == 64 )
	   {
///		   printf( "sending buffer length 64 chars\n" );
		   bulk_transfer_status = Tran_Bulk_Data( buffer, (ULONG) 64 );  //return FALSE if failed
	       if ( bulk_transfer_status == SUCCESS )
		   {
	          buffers_sent_count++;
	          total_char_count += buffer_char_count;   // only update with actually sent
	          buffer_char_count = 0;   //reset for new buffer
		   }
	       else   // ( bulk_transfer_status == FAILED )
		   {
	          buffer_char_count = 0;   // so do not try to send again. DO NOT REMOVE
		      printf( "\n64 char transfer failure\n" );
	          break;
		   }
	   }
   }
///   printf( "EOF or transfer failure\n" );
   if ( buffer_char_count )		// only send last part buffer here. Never do resends.
   {
	   bulk_transfer_status = Tran_Bulk_Data( buffer, buffer_char_count );
	   if ( bulk_transfer_status == SUCCESS )
	   {
	      buffers_sent_count++;
	      total_char_count += buffer_char_count;   // only update with actually sent
	   }
	   else   // ( bulk_transfer_status == FAILED )
	   {
		  printf( "\n%d char transfer failure\n", buffer_char_count );
	      buffer_char_count = 0;   // so do not try to send again. DO NOT REMOVE
	   }
   }

   fclose( fp );
   /* need \n on print summaries if tran_bulk_data() prints number times called */

   if ( bulk_transfer_status == SUCCESS )   // final and therefore all buffers sent OK
   {
      printf( "\nDOWNLOAD COMPLETED SUCCESSFULLY: %d chars downloaded in %d bulk transfers\n",
											total_char_count, buffers_sent_count );
      return 0;
   }
   else		//last buffer send failed, process terminated.
   {
      printf( "\nDOWNLOAD TERMINATED WITH FAILURE: %d chars downloaded in %d bulk transfers\n", total_char_count,
														buffers_sent_count );
	  return -1;
   }
 

}
/*----------------------------------------------------------------------------+
|                                                                             |
|                             A B O R T _ P I P E                             |
|                                                                             |
+-----------------------------------------------------------------------------*/

Abort_Pipe()
{
int	pipenum = 0;	//want endpout 1 out
	// Open the driver
	if (bOpenDriver (&hDevice, pcDriverName) != TRUE)
	{
		printf( "Failed to Open Driver\n");
		hDevice = NULL;
	}
	if (hDevice != NULL) 
	{
		bResult = DeviceIoControl (hDevice,
						IOCTL_Ezusb_ABORTPIPE,
						&pipenum,
						sizeof(ULONG),
						NULL,
						0,
						(unsigned long *)&nBytes,
						NULL);
	}/* if valid driver handle */
				
	CloseHandle (hDevice); // Close the handle
}

/*----------------------------------------------------------------------------+
|                                                                             |
|                                   M A I N                                   |
|                                                                             |
+-----------------------------------------------------------------------------*/

main( int argc, char **argv )
{
char	filename[256];
int		i;
int		status;
char    buffer[32];
ULONG   buflen = 32;

	if ( argc > 1 )
	{
		strcpy( pcDriverName, *(argv + 1) );
		pcDriverName[ strlen(pcDriverName) ] = '\0';
	}
	else
	{
		printf( "device name missing - can not open device\n" );
		exit(-1);
	}
	if ( argc > 2 )
	{
		strcpy( filename, *(argv + 2) );
		filename[ strlen(filename) ] = '\0';
	}
	else
	{
		printf( "download file name missing - can not proceed\n" );
		exit(-1);
	}
	printf( "Downloading file %s\n", filename );
//    pcDriverName[7] = '\0';		//make sure ezusb-0 is null terminated
	if (bOpenDriver (&hDevice, pcDriverName) == TRUE)
	{
		printf( "opened handle to %s\n", pcDriverName ); //Ezusb-0\n" );
		status = Send_File_Data( filename ); //		Tran_Bulk_Data();
		if ( argc > 3  && !status )
		{
		   for ( i = 3 ; i < argc && !status ; i++ )
		   {	
		      strcpy( filename, *(argv + i) );
		      filename[ strlen(*(argv+i)) ] = '\0';
			  printf( "Sending file %s\n", filename );
        	  status = Send_File_Data( filename ); //		Tran_Bulk_Data();
		   }
		}

		printf( "attempting to read\n" );
		status = Recv_Bulk_Data( buffer, buflen );   //Read_Back();

        CloseHandle (hDevice);
		exit( buffer[0] );
	}
	else
	{
		printf( "Unable to open device\n" );
		exit(-1);
	}
}

